/// <reference types="node" />
import { StreamClient, OnUploadProgress, RefreshUrlAPIResponse } from './client';
export declare class StreamFileStore {
    client: StreamClient;
    token: string;
    constructor(client: StreamClient, token: string);
    /**
     * upload a File instance or a readable stream of data
     * @link https://getstream.io/docs/files_introduction/?language=js#upload
     * @param {File|Buffer|NodeJS.ReadStream|string} uri - File object or stream or URI
     * @param {string} [name] - file name
     * @param {string} [contentType] - mime-type
     * @param {function} [onUploadProgress] - browser only, Function that is called with upload progress
     * @return {Promise<FileUploadAPIResponse>}
     */
    upload(uri: string | File | Buffer | NodeJS.ReadStream, name?: string, contentType?: string, onUploadProgress?: OnUploadProgress): Promise<import("./client").FileUploadAPIResponse>;
    /**
     * delete an uploaded file
     * @link https://getstream.io/docs/files_introduction/?language=js#delete
     * @param {string} uri
     */
    delete(uri: string): Promise<import("./client").APIResponse>;
    /**
     * Explicitly refresh CDN urls for uploaded files on the Stream CDN (only needed for files on the Stream CDN).
     * Note that Stream CDN is not enabled by default, if in doubt please contact us.
     * @param  {string} uri full uploaded file url that needs to be refreshed
     * @return {Promise<RefreshUrlAPIResponse>}
     */
    refreshUrl(uri: string): Promise<RefreshUrlAPIResponse>;
}
//# sourceMappingURL=files.d.ts.map